#[derive(Default, Clone)]
#[allow(clippy::struct_excessive_bools)]
pub struct Options {
    pub check: bool,
    pub skip_installed: bool,
    pub skip_ahead: bool,
    pub no_deps: bool,
    pub no_checksum: bool,
    pub generate: bool,
    pub gpg: bool,
    pub no_confirm: bool,
    pub read_file: Option<String>,
    pub category: bool,
    pub maintainer: bool,
    pub with_deps: bool,
}
