use crate::config::get_config;
use crate::utils::read_and_print;

/// Reads and prints the contents of the sbpkg configuration file.
///
/// # Exits
///
/// Exits with code 1 if the configuration file is not found.
pub fn show_config() {
    let cfg = get_config();
    read_and_print(&cfg.config_file, None);
}
